% Plots SQG trace at the end of iterations. This function is produced by
% modifying PlotSQGTrace to SQG_v05 requirements

function PlotSQGTrace_v02( NumProblems, ProblemNames, PlotFunctionAvg, PlotFunctionObs, ...
    RefFunctionValue, PlotVariableNumber, RefVariablesValues, PlotConstraintsViolation, ...
    iter, bufF, bufFAvg, bufVar, bufVarAvg, ConNumber, bufConV, bufConVAvg, ConRef, Secondaries, SaveFigures )

figureCounter=0;
X=1:1:iter;
XX=0:1:iter;

for i=1:NumProblems
    
    if PlotFunctionAvg(i)
        figureCounter=figureCounter+1;
        figure(figureCounter)
        line(X,bufFAvg(i,:),'Color','b');       
        if ~isempty(RefFunctionValue{i})
            line([1 iter],[RefFunctionValue{i} RefFunctionValue{i}],'Color','r');
            title(['Moving average (blue) and reference value (red) of function ' ...
                ProblemNames{i}]);
        else
            title(['Moving average of function ' ProblemNames{i}]);
        end
        if ~isempty(SaveFigures)
            FName=[SaveFigures '_' num2str(i) '_FA'];
            saveas(gcf,[FName '.fig']);
%            saveas(gcf,[FName '.emf']);
            saveas(gcf,[FName '.png']);
        end
        if PlotFunctionObs
            figureCounter=figureCounter+1;
            figure(figureCounter)
            line(X,bufF(i,:),'Color','b');
            line(X,bufFAvg(i,:),'Color','r'); 
            title(['Observations (blue) and moving average (red) of function ' ProblemNames{i}])
            if ~isempty(SaveFigures)
                FName=[SaveFigures '_' num2str(i) '_FO'];
                saveas(gcf,[FName '.fig']);
%                saveas(gcf,[FName '.emf']);
                saveas(gcf,[FName '.png']);
            end
        end      
    end
    
    for j=1:length(PlotVariableNumber{i})
        figureCounter=figureCounter+1;
        figure(figureCounter);
        line(XX,squeeze(bufVar(i,PlotVariableNumber{i}(j),:)),'Color','b')
        line(XX,squeeze(bufVarAvg(i,PlotVariableNumber{i}(j),:)),'Color','k')
        if ~isempty(RefVariablesValues{i})
            line([0 iter],[RefVariablesValues{i}(j) RefVariablesValues{i}(j)],'Color','r')
            title(['Variable ' num2str(PlotVariableNumber{i}(j)) ' of problem ' ...
                ProblemNames{i} ' (blue) with average (black) and reference (red)'])
        else
            title(['Variable ' num2str(PlotVariableNumber{i}(j)) ' of problem ' ...
                ProblemNames{i} ' (blue) with average (black)'])
        end
        if ~isempty(SaveFigures)
            FName=[SaveFigures '_' num2str(i) '_V' num2str(PlotVariableNumber{i}(j))];
            saveas(gcf,[FName '.fig']);
%            saveas(gcf,[FName '.emf']);  
            saveas(gcf,[FName '.png']);
        end
    end
    
    if PlotConstraintsViolation(i)
        for j=1:ConNumber(i)
            figureCounter=figureCounter+1;
            figure(figureCounter);
            line(X,squeeze(bufConV(i,j,:)),'Color','b')
            line(X,squeeze(bufConVAvg(i,j,:)),'Color','k')
            line([0 iter],[0 0],'Color','r')
            title(['Estimate of violation of constraint ' ProblemNames{Secondaries(ConRef(j))} ' of problem ' ...
                ProblemNames{i} ' (blue) with average (black)'])
            if ~isempty(SaveFigures)
                FName=[SaveFigures '_' num2str(i) '_C' num2str(j)];
                saveas(gcf,[FName '.fig']);
%                saveas(gcf,[FName '.emf']);
                saveas(gcf,[FName '.png']);
            end
        end     
    end
    
end


