% Plots SQG trace at the end of iterations. This function is produced by
% modifying PlotSQGTrace_v02 to SQG_v05_1 requirements

function PlotSQGTrace_v03( NumProblems, ProblemNames, PlotFunctionAvg, PlotFunctionObs, ...
    RefFunctionValue, PlotVariableNumber, RefVariablesValues, PlotConstraintsViolation, ...
    iter, bufF, bufFAvg, bufVar, bufVarAvg, ConNumber, bufConV, bufConVAvg, ConRef, ...
    Secondaries, SaveFigures, pathLength, PlotType, PlotAgainstVariables )

figureCounter=0;
X=1:1:iter;
XX=0:1:iter;
XXX=[1 iter];
XXXX=[0 iter];
Xlabel='Iterations';

for i=1:NumProblems
    
    if PlotFunctionAvg(i)
        if PlotType == 0
            X=pathLength(1:iter,i,PlotAgainstVariables(i))/pathLength(2,i,PlotAgainstVariables(i));
            XXX=[X(1) X(end)];
            Xlabel='Normalized accumulated stepsize';
        end
        figureCounter=figureCounter+1;
        figure(figureCounter)
        line(X,bufFAvg(i,:),'Color','b');       
        if ~isempty(RefFunctionValue{i})
            line(XXX,[RefFunctionValue{i} RefFunctionValue{i}],'Color','r');
            title(['Moving average (blue) and reference value (red) of function ' ...
                ProblemNames{i}]);
        else
            title(['Moving average of function ' ProblemNames{i}]);
        end
        xlabel(Xlabel);
        if ~isempty(SaveFigures)
            FName=[SaveFigures '_' num2str(i) '_FA'];
            saveas(gcf,[FName '.fig']);
            saveas(gcf,[FName '.emf']);         
        end
        if PlotFunctionObs
            figureCounter=figureCounter+1;
            figure(figureCounter)
            line(X,bufF(i,:),'Color','b');
            line(X,bufFAvg(i,:),'Color','r'); 
            title(['Observations (blue) and moving average (red) of function ' ProblemNames{i}])
            xlabel(Xlabel);
            if ~isempty(SaveFigures)
                FName=[SaveFigures '_' num2str(i) '_FO'];
                saveas(gcf,[FName '.fig']);
                saveas(gcf,[FName '.emf']);         
            end
        end      
    end
    
    for j=1:length(PlotVariableNumber{i})
        if PlotType == 0
            XX=pathLength(1:end,i,PlotVariableNumber{i}(j))/pathLength(2,i,PlotVariableNumber{i}(j));
            XXXX=[XX(1) XX(end)];
        end
        figureCounter=figureCounter+1;
        figure(figureCounter);
        line(XX,squeeze(bufVar(i,PlotVariableNumber{i}(j),:)),'Color','b')
        line(XX,squeeze(bufVarAvg(i,PlotVariableNumber{i}(j),:)),'Color','k')
        if ~isempty(RefVariablesValues{i})
            line(XXXX,[RefVariablesValues{i}(j) RefVariablesValues{i}(j)],'Color','r')
            title(['Variable ' num2str(PlotVariableNumber{i}(j)) ' of problem ' ...
                ProblemNames{i} ' (blue) with average (black) and reference (red)'])
        else
            title(['Variable ' num2str(PlotVariableNumber{i}(j)) ' of problem ' ...
                ProblemNames{i} ' (blue) with average (black)'])
        end
        xlabel(Xlabel);
        if ~isempty(SaveFigures)
            FName=[SaveFigures '_' num2str(i) '_V' num2str(PlotVariableNumber{i}(j))];
            saveas(gcf,[FName '.fig']);
            saveas(gcf,[FName '.emf']);         
        end
    end
    
    if PlotConstraintsViolation(i)
        if PlotType == 0
            X=pathLength(1:iter,i,PlotAgainstVariables(i))/pathLength(2,i,PlotAgainstVariables(i));
            XXX=[X(1) X(end)];
        end
        for j=1:ConNumber(i)
            figureCounter=figureCounter+1;
            figure(figureCounter);
            line(X,squeeze(bufConV(i,j,:)),'Color','b')
            line(X,squeeze(bufConVAvg(i,j,:)),'Color','k')
            line(XXX,[0 0],'Color','r')
            title(['Estimate of violation of constraint ' ProblemNames{Secondaries(ConRef(j))} ' of problem ' ...
                ProblemNames{i} ' (blue) with average (black)'])
            xlabel(Xlabel);
            if ~isempty(SaveFigures)
                FName=[SaveFigures '_' num2str(i) '_C' num2str(j)];
                saveas(gcf,[FName '.fig']);
                saveas(gcf,[FName '.emf']);         
            end
        end     
    end
    
end


