% assigns default values to SQG parameters and changes them according to
% parAlg. This is cell array, which every element is a cell with two components:
% name of parameter and parameter value
% This version is created for StochasticGradients_v05_1, its predecessor is
% SQGParameters_v03

function [illegalInput, NumProblems, nVars, ProblemNames, ProblemStates, ProblemType, ...
    DPrimary, NSecondaries, Secondaries,DSecondaries, SecondariesAveraging, IsSecondary, ...
    iter, iavg, stepScale, step0, schedule, stepmult, ProblemScale, FDiff, gamma, ...
    TraceAveraging, MaxVarChange, VarRange, PlotFunctionAvg, PlotFunctionObs, RefFunctionValue, ...
    PlotVariableNumber, RefVariablesValues, SaveFileName, RunComment, randInitFlag, ...
    ConNumber, ConTypes, ConRHS, ConRef, EstimateGradient, ConProcessing, ConPenalty, ...
    PlotConstraintsViolation, SaveFigures, KeepTrack, PlotType, PlotAgainstVariables] ...
    = SQGParameters_v04( parAlg, x0 )

illegalInput=0;

% output in the case of abort

NumProblems=[];
nVars=[];
ProblemNames=[];
ProblemStates=[];
ProblemType=[];
DPrimary=[];
NSecondaries=[];
Secondaries=[];
DSecondaries=[];
SecondariesAveraging=[];
IsSecondary=[];
iter=[];
iavg=[];
stepScale=[];
step0=[];
schedule=[];
stepmult=[];
ProblemScale=[];
FDiff=[];
gamma=[];
TraceAveraging=[];
MaxVarChange=[];
VarRange=[];
PlotFunctionAvg=[];
PlotFunctionObs=[];
RefFunctionValue=[];
PlotVariableNumber=[];
RefVariablesValues=[];
SaveFileName=[];
RunComment=[];
randInitFlag=[];
ConNumber=[];
ConTypes=[];
ConRHS=[];
ConRef=[];
EstimateGradient=[];
ConProcessing=[];
ConPenalty=[];
PlotConstraintsViolation=[];
SaveFigures=[];
KeepTrack=[];
PlotType=[];
PlotAgainstVariables=[];

% Rarely changed defaults invisible to user

AccuracyToFinDiff                = 0.5;
FractionForTraceAveraging        = 0.1;
BigNumber                        = 1e10;

NoExplicitProblemNames=1;
NoExplicitProblemDimensions=1;
ConstraintsBlockAlreadyEncountered=0;
UndefinedProblemTypes=1;
PenaltyNotSet=1;
ProblemCorrespondence=[];

% Looking for problem names 

for i=1:length(parAlg)
    if strcmp(parAlg{i}{1},'ProblemNames')
        ProblemNames=parAlg{i}(2:end);
        NoExplicitProblemNames=0;
        NumProblems=length(ProblemNames);
        for ii=1:length(ProblemNames)
            if ~ischar(ProblemNames{ii})
               disp('SQG ERROR 050: Problem name must be a string');
               illegalInput=1;
               return 
            end
        end
        break
    end
end

% Default number of problems

NN=size(x0);
if NoExplicitProblemNames
    NumProblems=NN(1);
elseif NumProblems ~= NN(1)
    disp('SQG ERROR 041: Mismatch between number of problem names and number of initial points');
    illegalInput=1;
    return
end

% Problem types

for i=1:length(parAlg)
    PName=parAlg{i}{1};
    PValue=parAlg{i}(2:end);
    if strcmp(PName,'ProblemTypes')      % for each problem
     UndefinedProblemTypes=0;
     ProblemType=zeros(1,NumProblems);
     if length(PValue) == 1
        if strcmp(PValue{1},'max')
            ProblemType=-ones(1,NumProblems);
        elseif strcmp(PValue{1},'min')
            ProblemType=ones(1,NumProblems);
        elseif strcmp(PValue{1},'estimate')
            ProblemType=zeros(1,NumProblems);
        else
            disp(['SQG ERROR 001: unknown problem type ' PValue{1}]);
            illegalInput=1;
            return
        end
     else
        ProblemType=zeros(1,NumProblems);
        if length(PValue) ~= NumProblems
            disp('SQG ERROR 028: Incorrect length of user defined problem types');
            illegalInput=1;
            return
        end
        for j=1:NumProblems
            if strcmp(PValue{j},'max')
                ProblemType(j)=-1;
            elseif strcmp(PValue{j},'min')
                ProblemType(j)=1;
            elseif strcmp(PValue{j},'estimate')
                ProblemType(j)=0;
            else
                disp(['SQG ERROR 001: unknown problem type ' PValue{j}]);
                illegalInput=1;
                return
            end
        end
     end
    end
end
if sum(ProblemType==0) == NumProblems
    disp('SQG ERROR 401: No optimization problems are declared ');
    illegalInput=1;
    return
end
         
% Looking for problem dimensions
 
nVars=ones(1,NumProblems);
for i=1:length(parAlg)
    if strcmp(parAlg{i}{1},'ProblemDimensions')
        NoExplicitProblemDimensions=0;
        ProblemDimensions=parAlg{i}(2:end);
        if length(ProblemDimensions) == 1
            if isnumeric(ProblemDimensions{1}) && ~isinf(ProblemDimensions{1}) && ...
                (ProblemDimensions{1} > 0) && (floor(ProblemDimensions{1}) == ProblemDimensions{1})
            nVars=ProblemDimensions{1}*nVars;
            if sum(ProblemType==0) > 0
                disp('SQG ERROR 460: Attempt to assign optimization variables to estimation problem');
                illegalInput=1;
                return
            end
            else
                disp('SQG ERROR 060: Dimension of optimization problem must be positive finite integer');
                illegalInput=1;
                return
            end
        elseif length(ProblemDimensions) == NumProblems
            for j=1:NumProblems
                if isnumeric(ProblemDimensions{j}) && ~isinf(ProblemDimensions{j}) && ...
                    (ProblemDimensions{j} > 0) && (floor(ProblemDimensions{j}) == ProblemDimensions{j})
                nVars(j)=ProblemDimensions{j};
                elseif ProblemType(j) == 0
                    if ProblemDimensions{j} > 0
                        disp('SQG ERROR 460a: Attempt to assign optimization variables to estimation problem');
                        illegalInput=1;
                        return
                    else
                        nVars(j)=0;
                    end                    
                else
                    disp('SQG ERROR 060a: Dimension of optimization problem must be positive finite integer');
                    illegalInput=1;
                    return
                end
            end
        else 
            disp('SQG ERROR 050a: Number of set problem dimensions is different from the number of problems');
            illegalInput=1;
            return
        end
        break
    end
end

% Check consistency of problem dimensions

if ~NoExplicitProblemDimensions
    if max(nVars) ~= NN(2)
        disp('SQG ERROR 441: Mismatch between explicitly declared problem dimensions and dimension of initial points');
        illegalInput=1;
        return
    end
end

% Default problem dimensions and check of x0

if NoExplicitProblemDimensions
    nVars=nVars*NN(2);
end

for i=1:NumProblems
    for j=1:nVars(i)
        if isinf(x0(i,j))
            if NoExplicitProblemDimensions
                nVars(i)=j-1;
                break
            else
                disp('SQG ERROR 061: Infinite values in initial point');
                illegalInput=1;
                return
            end
        end
    end
end

% Defaults

if NoExplicitProblemNames
    ProblemNames=cell(1,NumProblems);
    for i=1:NumProblems
        ProblemNames{i}=['function ' num2str(i)];
    end
    if NumProblems == 1
       ProblemNames{1} = 'function';
    end
end

if UndefinedProblemTypes
    ProblemType                     = ones(1,NumProblems);
end
iter                            = 5000;
stepScale                       = cell(1,NumProblems);
for i=1:NumProblems
    stepScale{i}=ones(1,nVars(i));
end
FractionLastAveragingIterations = 0.1*ones(1,NumProblems);
step0                           = ones(1,NumProblems); % in the later versions compute default initial stepsize from problem characteristics
schedule                        = cell(1,NumProblems);
for i=1:NumProblems
    schedule{i}=[50 100 200 400 800 1600 3200 6400 12500 25000 50000 100000 ...
                                   200000 400000 800000 1500000 3000000 6000000];
end
stepmult                        = 0.55*ones(1,NumProblems);
ProblemScale                    = cell(1,NumProblems);
AccuracyVariables               = cell(1,NumProblems);
for i=1:NumProblems
    AccuracyVariables{i}=0.1*ones(1,nVars(i));
end
gamma                           = 0.001*ones(1,NumProblems);
MaxVarChange                    = 0.1*ones(1,NumProblems);
VarRange                        = cell(1,NumProblems);
for i=1:NumProblems
    VarRange{i}=BigNumber*ones(1,nVars(i));
end
PlotFunctionAvg                 = zeros(1,NumProblems);
PlotFunctionObs                 = 0;
RefFunctionValue                = cell(1,NumProblems);
PlotVariableNumber              = cell(1,NumProblems);
RefVariablesValues              = cell(1,NumProblems);
PlotConstraintsViolation        = zeros(1,NumProblems);
PlotConBuf                      = [];
SaveFileName                    = [];
RunComment                      = [];
randInitFlag                    = 0;
ProblemStates                   = zeros(1,NumProblems);
DPrimary                        = ones(1,NumProblems); % 1 correspond to forward finite differences
TraceAveraging                  = zeros(NumProblems,3); % zero corresponds to moving average
SecondariesAveraging            = zeros(NumProblems,3);
ConNumber                       = zeros(1,NumProblems);
ConFunctions                    = zeros(NumProblems,NumProblems-1);
ConTypes                        = zeros(NumProblems,NumProblems-1);
ConRHS                          = zeros(NumProblems,NumProblems-1);
ConDirection                    = zeros(NumProblems,NumProblems-1);
ConRef                          = zeros(NumProblems,NumProblems-1);
NSecondaries                    = zeros(1,NumProblems);
Secondaries                     = zeros(NumProblems,NumProblems-1);
DSecondaries                    = zeros(NumProblems,NumProblems-1);
IsSecondary                     = zeros(1,NumProblems);
EstimateGradient                = 1;
ConProcessing                   = zeros(NumProblems,2);
ConProcessing(:,1)              = 1;
ConPenalty                      = 5*ones(1,NumProblems);
SaveFigures                     = [];
KeepTrack                       = 1;
PlotType                        = 1;
PlotAgainstVariables            = ones(1,NumProblems);

% User defined values

DefaultFiniteDifferences=1;
DefaultTraceAveraging=1;
DefaultConstraintsAveraging=1;
DefaultProblemScales=1;
RefBuf=[];
PlotFunctionAvgBuf=[];
RefFunctionValueBuf=[];

for i=1:length(parAlg)
    PName=parAlg{i}{1};
    PValue=parAlg{i}(2:end);
    
    if strcmp(PName,'ProblemStates')
        ProblemStates = ones(1,NumProblems);
        if length(PValue) == 1
            if isnumeric(PValue{1}) && ~isinf(PValue{1}) && (PValue{1} > 0) ...
                    && (floor(PValue{1}) == PValue{1})
            ProblemStates=PValue{1}*ProblemStates;
            else
                disp('SQG ERROR 062: Number of problem states must be positive finite integer');
                illegalInput=1;
                return
            end
        elseif length(PValue) == NumProblems
            for j=1:NumProblems
                if isnumeric(PValue{j}) && ~isinf(PValue{j}) && ...
                    (PValue{j} > 0) && (floor(PValue{j}) == PValue{j})
                ProblemStates(j)=PValue{j};
                else
                    disp('SQG ERROR 062a: Number of problem states must be positive finite integer');
                    illegalInput=1;
                    return
                end
            end
        else 
            disp('SQG ERROR 062b: Number of set problem states is different from the number of problems');
            illegalInput=1;
            return
        end
        
    elseif strcmp(PName,'StepDirections')      % for each problem
         if length(PValue) == 1
            if strcmp(PValue{1},'grad')
                DPrimary=zeros(1,NumProblems);
            elseif strcmp(PValue{1},'forwardDiff')
                DPrimary=ones(1,NumProblems);
            else
                disp(['SQG ERROR 101: unknown direction type ' PValue{1}]);
                illegalInput=1;
                return
            end
        else
            if length(PValue) ~= NumProblems
                disp('SQG ERROR 102: Incorrect length of user defined direction types');
                illegalInput=1;
                return
            end
            for j=1:NumProblems
                if strcmp(PValue{j},'grad')
                    DPrimary(j)=0;
                elseif strcmp(PValue{j},'forwardDiff')
                    DPrimary(j)=1;
                else
                    disp(['SQG ERROR 101a: unknown problem type ' PValue{j}]);
                    illegalInput=1;
                    return
                end
            end
         end  
         
    elseif strcmp(PName,'Constraints')
        if ConstraintsBlockAlreadyEncountered
            disp('SQG ERROR 130: Second constraints block is not allowed');
            illegalInput=1;
            return
        else
            ConstraintsBlockAlreadyEncountered=1;
        end
        for j=1:length(PValue)
            [ P, F, T, RHS, D, WrongDefinition ] = ParseConstraint( PValue{j} );
            if WrongDefinition
                disp('SQG ERROR 131: Parser has encountered illegal definition of constraint');
                illegalInput=1;
                return
            end
            
            if ischar(P)
                NameNotFound=1;
                for k=1:NumProblems
                    if strcmp(P,ProblemNames{k})
                        NameNotFound=0;
                        CurrentProblem=k;
                        ConNumber(CurrentProblem)=ConNumber(CurrentProblem)+1;
                    end
                end
                if NameNotFound
                    disp('SQG ERROR 132: Nonexistent problem name is found in constraint definition');
                    illegalInput=1;
                    return
                end
            else
                if (P > 0) && (P <= NumProblems) && (floor(P) == P)
                    CurrentProblem=P;
                    ConNumber(CurrentProblem)=ConNumber(CurrentProblem)+1;
                else
                    disp('SQG ERROR 133: Nonexistent problem number is found in constraint definition');
                    illegalInput=1;
                    return
                end
            end
            
            if ischar(F)
                NameNotFound=1;
                for k=1:NumProblems
                    if strcmp(F,ProblemNames{k})
                        NameNotFound=0;
                        ConFunctions(CurrentProblem,ConNumber(CurrentProblem))=k;
                    end
                end
                if NameNotFound
                    disp('SQG ERROR 134: Nonexistent function name is found in constraint definition');
                    illegalInput=1;
                    return
                end
                if k == CurrentProblem
                    disp('SQG ERROR 136: Constraint function must be different from problem function');
                    illegalInput=1;
                    return
                end
            else
                if (F > 0) && (F <= NumProblems) && (floor(F) == F)
                    ConFunctions(CurrentProblem,ConNumber(CurrentProblem))=F;
                    if F == CurrentProblem
                        disp('SQG ERROR 136a: Constraint function must be different from problem function');
                        illegalInput=1;
                        return
                    end
                else
                    disp('SQG ERROR 135: Nonexistent function number is found in constraint definition');
                    illegalInput=1;
                    return
                end
            end
            
            ConTypes(CurrentProblem,ConNumber(CurrentProblem))=T;
            ConRHS(CurrentProblem,ConNumber(CurrentProblem))=RHS;
            ConDirection(CurrentProblem,ConNumber(CurrentProblem))=D;
                      
        end
        
    elseif strcmp(PName,'ConstraintsProcessing')      % for each problem
        ConProcessing=zeros(NumProblems,2);
        if length(PValue) == 1
            PV=cell(1,NumProblems);
            for j=1:NumProblems
                PV{j}=PValue{1};
            end
        elseif length(PValue) == NumProblems
            PV=PValue;
        else
            disp('SQG ERROR 352: Incorrect length of constraint processing types');
            illegalInput=1;
            return
        end
        for j=1:NumProblems
            buf=PV{j};
            if ischar(buf)
                buf={buf};
            elseif ~iscell(buf)
                disp('SQG ERROR 352a: Incorrect structure of constraint processing types');
                illegalInput=1;
                return
            end
            for k=1:length(buf)
                if strcmp(buf{k},'linearPenalty')
                    if ConProcessing(j,1) == 0
                        ConProcessing(j,1) = 1;
                    else
                        disp('SQG ERROR 352c: Penalty option defined twice');
                        illegalInput=1;
                        return
                    end
                elseif strcmp(buf{k},'quadraticPenalty')
                    if ConProcessing(j,1) == 0
                        ConProcessing(j,1) = 2;
                    else
                        disp('SQG ERROR 352d: Penalty option defined twice');
                        illegalInput=1;
                        return
                    end
                elseif strcmp(buf{k},'constraintPriority')
                    ConProcessing(j,2) = 1;
                else
                    disp(['SQG ERROR 352b: Unknown constraint processing type' buf{k}] );
                    illegalInput=1;
                    return
                end
            end
        end
         
         elseif strcmp(PName,'ConstraintsPenalty')   % for each problem and each variable      
            PValue=cell2mat(PValue);
            PenaltyNotSet=0;
        if isnumeric(PValue) && min(~isinf(PValue)) && (min(PValue) >= 0) 
            if length(PValue) ==1
                ConPenalty=PValue*ones(1,NumProblems);
            elseif length(PValue) == NumProblems
                ConPenalty=PValue;
            else
                disp('SQG ERROR 329: The length of the array of penalties mismatch the number of problems');
                illegalInput=1;
                return
            end                         
        else
            disp('SQG ERROR 003: The penalty coefficients must be positive numbers');
            illegalInput=1;
            return
        end
 
    elseif strcmp(PName,'NumberIterations')         % common
        PValue=cell2mat(PValue);
        if isnumeric(PValue) && ~isinf(PValue) && (length(PValue)==1) && (PValue > 0)        
            iter=PValue;
        else
            disp('SQG ERROR 002: Number of iterations must be positive scalar ');
            illegalInput=1;
            return
        end
      
        %  FractionLastAveragingIterations is the number of last iterations
        %  to average in order to get the problem solution
        
    elseif strcmp(PName,'FractionLastAveragingIterations') % for each problem
        PValue=cell2mat(PValue);
        if isnumeric(PValue) && min(~isinf(PValue)) && (min(PValue) >= 0) && (max(PValue) <= 1)
            if length(PValue) ==1
                FractionLastAveragingIterations=PValue*ones(1,NumProblems);
            elseif length(PValue) == NumProblems
                FractionLastAveragingIterations=PValue;
            else
                disp('SQG ERROR 029: The number of elements in array of fractions of the last averaging iterations mismatch the number of problems');
                illegalInput=1;
                return
            end                         
        else
            disp('SQG ERROR 003: The fraction of last averaging iterations must be scalar from interval [0,1]');
            illegalInput=1;
            return
        end
    
    elseif strcmp(PName,'StepScale')   % for each problem and each variable      
        if length(PValue) ~= NumProblems
            disp('SQG ERROR 030: Number of stepsize scales is different from the number of problems');
            illegalInput=1;
            return
        end
        for j=1:length(PValue)
            if isnumeric(PValue{j}) && min(~isinf(PValue{j})) && min(PValue{j} >= 0) && (length(PValue{j}) == nVars(j))
                stepScale{j}=PValue{j};
            else
                disp('SQG ERROR 004: Cells of step size scales must be nonnegative numeric arrays with length equal to the number of variables in a specific problem');
                illegalInput=1;
                return
            end
        end
        
    elseif strcmp(PName,'InitialStep')   % for each problem
        PValue=cell2mat(PValue);
        if isnumeric(PValue) && min(~isinf(PValue)) && min(PValue >= 0) 
            if length(PValue) ==1
                step0=PValue*ones(1,NumProblems);
            elseif length(PValue) == NumProblems
                step0=PValue;
            else
                disp('SQG ERROR 031: The number of elements in array of in initial stepsizes mismatches the number of problems');
                return
            end               
            illegalInput=0;
        else
            disp('SQG ERROR 005: Initial step size must be nonegative scalar');
            return
        end
    
    elseif strcmp(PName,'StepSchedule')     % for each problem
        if length(PValue) == 1
            if isnumeric(PValue{1}) && min(~isinf(PValue{1})) && min(PValue{1} > 0) && min((PValue{1}(2:end)-PValue{1}(1:end-1) > 0))        
                for j=1:NumProblems
                    schedule{j}=PValue{1};
                end
            else
                disp('SQG ERROR 006: Schedule for stepsize decrease must be a sequence of positive increasing numbers');
                illegalInput=1;
                return
            end
        elseif length(PValue) == NumProblems
            for j=1:NumProblems
                if isnumeric(PValue{j}) && min(~isinf(PValue{j})) && min(PValue{j} > 0) && min((PValue{j}(2:end)-PValue{j}(1:end-1) > 0))
                    schedule{j}=PValue{j};
                else
                    illegalInput=1;
                    disp('SQG ERROR 006: Schedule for stepsize decrease must be a sequence of positive increasing numbers');
                    return
                end
            end
        else
            disp('SQG ERROR 032: The number of stepsize schedules should be equal either to one or to the number of objectives ');
            illegalInput=1;
            return
        end
   
    elseif strcmp(PName,'StepMultiplier')   % for each problem
        PValue=cell2mat(PValue);
        if isnumeric(PValue) && min(~isinf(PValue)) && (min(PValue) >= 0) && (max(PValue) <= 1)
            if length(PValue) ==1
                stepmult=PValue*ones(1,NumProblems);
            elseif length(PValue) == NumProblems
                stepmult=PValue;
            else
                disp('SQG ERROR 033: The number of elements in array of stepsize multipliers mismatches the number of problems');
                illegalInput=1;
                return
            end               
        else
            disp('SQG ERROR 007: Multiplier for step size decrease must be nonnegative double array that does not exceed 1');
            illegalInput=1;
            return
        end
        
    elseif strcmp(PName,'ProblemScales')   % for each problem
        DefaultProblemScales=0;
        if length(PValue) == 1
            if isnumeric(PValue{1}) && min(~isinf(PValue{1})) && (min(PValue{1}) >= 0) && (length(PValue{1}) == NumProblems)
                for ii=1:NumProblems
                    ProblemScale{ii}=PValue{1}(ii); 
                end
            else
                disp('SQG ERROR 034: Fixed problems scales must be nonnegative double array with the length equal to the number of problems');
                illegalInput=1;
                return
            end
        elseif length(PValue) == NumProblems
            for ii=1:NumProblems
                if isnumeric(PValue{ii}) && min(~isinf(PValue{ii})) && (min(PValue{ii}) >= 0) 
                    ProblemScale{ii}=PValue{ii};
                else
                    disp('SQG ERROR 034a: Variable problems scales must be nonnegative double arrays ');
                    illegalInput=1;
                    return
                end
            end
        else
            disp('SQG ERROR 051: The number of problem scale arrays is different from the number of problems');
            illegalInput=1;
            return
        end
           
    elseif strcmp(PName,'AccuracyVariables') % for each problem and each variable. It can be double scalar, double array or cell array of double arrays
        if (length(PValue) == 1) 
            if (length(PValue{1}) == 1) && isnumeric(PValue{1}) && ~isinf(PValue{1}) && (PValue{1} > 0) 
                for j=1:NumProblems
                    AccuracyVariables{j}=PValue{1}*ones(1,nVars(j));
                end
            elseif (length(PValue{1}) == NumProblems) && isnumeric(PValue{1}) && min(~isinf(PValue{1})) && (min(PValue{1}) > 0) 
                for j=1:NumProblems
                    AccuracyVariables{j}=PValue{1}(j)*ones(1,nVars(j));
                end
            else
                disp('SQG ERROR 008a: Length of a single accuracy array does not match the number of problems');
                illegalInput=1;
                return
            end
        elseif length(PValue) == NumProblems
            for j=1:NumProblems
                if (length(PValue{j}) == nVars(j)) && isnumeric(PValue{j}) && min(~isinf(PValue{j})) && (min(PValue{j}) > 0) 
                    AccuracyVariables{j}=PValue{j};
                else
                    disp('SQG ERROR 008: Expected accuracy cells must contain array of nonnegative numbers with the length equal to the number of variables in corresponding problem');
                    illegalInput=1;
                    return
                end
            end
        else
            disp('SQG ERROR 035: Mismatch between number of problems and composition of accuracy array for problem variables');
            illegalInput=1;
            return
        end
        
    elseif strcmp(PName,'StepFiniteDifferences') % for each problem and each variable. It can be double scalar, double array or cell array of double arrays
        DefaultFiniteDifferences=0;
        FDiff=cell(1,NumProblems);
        if (length(PValue) == 1) 
            if (length(PValue{1}) == 1) && isnumeric(PValue{1}) && ~isinf(PValue{1}) && (PValue{1} > 0) 
                for j=1:NumProblems
                    FDiff{j}=PValue{1}*ones(1,nVars(j));
                end
            elseif (length(PValue{1}) == NumProblems) && isnumeric(PValue{1}) && min(~isinf(PValue{1})) && (min(PValue{1}) > 0) 
                for j=1:NumProblems
                    FDiff{j}=PValue{1}(j)*ones(1,nVars(j));
                end
            else
                disp('SQG ERROR 009a: Length of a single finite difference array does not match the number of problems');
                illegalInput=1;
                return
            end
        elseif length(PValue) == NumProblems
            for j=1:NumProblems
                if (length(PValue{j}) == nVars(j)) && isnumeric(PValue{j}) && min(~isinf(PValue{j})) && (min(PValue{j}) > 0) 
                    FDiff{j}=PValue{j};
                else
                    disp('SQG ERROR 009: Finite differences cells must contain array of nonnegative numbers with the length equal to the number of variables in corresponding problem');
                    illegalInput=1;
                    return
                end
            end
        else
            disp('SQG ERROR 036: Mismatch between number of problems and composition of problem differences array for problem variables');
            illegalInput=1;
            return
        end

    elseif strcmp(PName,'AveragingFiniteDifferences')   % for each problem
        PValue=cell2mat(PValue);
        if isnumeric(PValue) && min(~isinf(PValue)) && (min(PValue) >= 0) && (max(PValue) <= 1)
            if length(PValue) ==1
                gamma=PValue*ones(1,NumProblems);
            elseif length(PValue) == NumProblems
                gamma=PValue;
            else
                disp('SQG ERROR 037: The number of elements in array of averaging multipliers for finite differences mismatches the number of problems');
                illegalInput=1;
                return
            end               
        else
            disp('SQG ERROR 010: Averaging multipliers for finite differences must be nonnegative and not exceed 1');
            illegalInput=1;
            return
        end  

    elseif strcmp(PName,'TraceAveraging')   % for each problem, in the future decide about the optimal value depending on variability?
        DefaultTraceAveraging=0;
        if length(PValue) == 1 
            PV=cell(1,NumProblems);
            for j=1:NumProblems
                PV{j}=PValue{1};
            end
        elseif length(PValue) == NumProblems
            PV=PValue;
        else
            disp('SQG ERROR 038: Incorrect number of triples of trace averaging parameters');   
            illegalInput=1;
            return
        end
        for j=1:NumProblems
            buf=PV{j};
            if isnumeric(buf) && (length(buf) == 3) && (buf(1) > 0) && (min(buf(2),buf(3)) >= 0)
                TraceAveraging(j,:)=buf;
            else
                disp('SQG ERROR 038a: Incorrect structure of trace averaging parameters');   
                illegalInput=1;
                return
            end
        end
        
    elseif strcmp(PName,'ConstraintsAveraging')   % for each problem, in the future decide about the optimal value depending on variability?
        DefaultConstraintsAveraging=0;
        if length(PValue) == 1 
            PV=cell(1,NumProblems);
            for j=1:NumProblems
                PV{j}=PValue{1};
            end
        elseif length(PValue) == NumProblems
            PV=PValue;
        else
            disp('SQG ERROR 238: Incorrect number of triples of constraints averaging parameters');   
            illegalInput=1;
            return
        end
        for j=1:NumProblems
            buf=PV{j};
            if isnumeric(buf) && (length(buf) == 3) && (buf(1) > 0) && (min(buf(2),buf(3)) >= 0)
                SecondariesAveraging(j,:)=buf;
            else
                disp('SQG ERROR 238a: Incorrect structure of constraints averaging parameters');   
                illegalInput=1;
                return
            end
        end  

    elseif strcmp(PName,'MaximalVariableChange')   % for each problem, in the future use it to define initial stepsize?
        PValue=cell2mat(PValue);
        if isnumeric(PValue) && min(~isinf(PValue)) && (min(PValue) >= 0) 
            if length(PValue) ==1
                MaxVarChange=PValue*ones(1,NumProblems);
            elseif length(PValue) == NumProblems
                MaxVarChange=PValue;
            else
                disp('SQG ERROR 039: The number of elements in array of bounds for relative variable change mismatches the number of problems');
                illegalInput=1;
                return
            end                          
        else
            disp('SQG ERROR 013: Maximal allowed change of variables relative to their ranges must be nonnegative doubles');
            illegalInput=1;
            return
        end

    elseif strcmp(PName,'VariablesRanges') % for each problem and each variable. It can be double scalar, double array or cell array of double arrays
        if (length(PValue) == 1) 
            if (length(PValue{1}) == 1) && isnumeric(PValue{1}) && ~isinf(PValue{1}) && (PValue{1} > 0) 
                for j=1:NumProblems
                    VarRange{j}=PValue{1}*ones(1,nVars(j));
                end
            elseif (length(PValue{1}) == NumProblems) && isnumeric(PValue{1}) && min(~isinf(PValue{1})) && (min(PValue{1}) > 0) 
                for j=1:NumProblems
                    VarRange{j}=PValue{1}(j)*ones(1,nVars(j));
                end
            else
                disp('SQG ERROR 040a: Mismatch between number of problems and composition of ranges array for problem variables');
                illegalInput=1;
                return
            end
        elseif length(PValue) == NumProblems
            for j=1:NumProblems
                if (length(PValue{j}) == nVars(j)) && isnumeric(PValue{j}) && min(~isinf(PValue{j})) && (min(PValue{j}) > 0) 
                    buf=PValue{j};
                    nbuf=length(buf);
                    if nbuf(2) == 1
                        buf=buf';
                    end
                    VarRange{j}=buf;
                else
                    disp('SQG ERROR 014: Ranges of variables must be array of positive numbers with the length equal to the number of variables');
                    illegalInput=1;
                    return
                end
            end
        else
            disp('SQG ERROR 040: Mismatch between number of problems and composition of ranges array for problem variables');
            illegalInput=1;
            return
        end
        
    elseif strcmp(PName,'EstimateGradient') % common
        if strcmp(PValue{1},'yes')
            EstimateGradient=1;
        elseif strcmp(PValue{1},'no')
            EstimateGradient=0;
        else
            disp('SQG ERROR 216: EstimateGradient takes values ''yes'' or ''no'' ');
            illegalInput=1;
            return
        end
        
    elseif strcmp(PName,'PlotType') % common
        if strcmp(PValue{1},'Iteration')
            PlotType=1;
        elseif strcmp(PValue{1},'Path')
            PlotType=0;
        else
            disp('SQG ERROR 4001: PlotType takes values ''Iteration'' or ''Path'' ');
            illegalInput=1;
            return
        end
        
    elseif strcmp(PName,'PlotAgainstVariables') % common
        if length(PValue) == 1
            buf=PValue{1};
            if isnumeric(buf) && (length(buf) == NumProblems)
                PlotAgainstVariables=buf;
            else
                disp('SQG ERROR 4002: PlotAgainstVariables has illegal type or length ');
                illegalInput=1;
                return
            end
        elseif length(PValue) == NumProblems
            PlotAgainstVariables = zeros(1,NumProblems);
            for j=1:length(PValue)
                buf=PValue{j};
                if isnumeric(buf) && (length(buf) == 1) && (buf > 0) && (buf <= nVars(j) )
                    PlotAgainstVariables(j)=buf;
                else
                    disp('SQG ERROR 4004: PlotAgainstVariables: illegal value is found');
                    illegalInput=1;
                    return
                end
            end
        else
            disp('SQG ERROR 4003: PlotAgainstVariables has illegal length ');
            illegalInput=1;
            return
        end    
        
    elseif strcmp(PName,'KeepTrack') % common
        if strcmp(PValue{1},'yes')
            KeepTrack=1;
        elseif strcmp(PValue{1},'no')
            KeepTrack=0;
        else
            disp('SQG ERROR 516: KeepTrack takes values ''yes'' or ''no'' ');
            illegalInput=1;
            return
        end

    elseif strcmp(PName,'PlotFunctionAverage')  % common
        if (NumProblems == 1) && (length(PValue) == 1)
            if strcmp(PValue{1},'yes')
                PlotFunctionAvgBuf=1;
            elseif strcmp(PValue{1},'no')
                PlotFunctionAvgBuf=[];
            else
                disp('SQG ERROR 015: PlotFunctionAverage with a single objective takes values ''yes'' or ''no'' ');
                illegalInput=1;
                return
            end  
        else    
            PlotFunctionAvgBuf=zeros(1,length(PValue));
            for j=1:length(PValue)
                if isnumeric(PValue{j}) && min(~isinf(PValue{j})) && min(floor(PValue{j}) == PValue{j}) && ...
                    (max(PValue{j}) <= NumProblems) && (min(PValue{j}) > 0)
                    PlotFunctionAvgBuf(j)=PValue{j};
                elseif ischar(PValue{j})
                    for k=1:NumProblems
                        if strcmp(PValue{j},ProblemNames{k})
                            PlotFunctionAvgBuf(j)=k;
                            break
                        end
                    end
                else
                    disp('SQG ERROR 042: Illegal request for plotting function averages');
                    illegalInput=1;
                    return
                end
            end
            if sum(PlotFunctionAvgBuf >0) < length(PValue)
                disp('SQG WARNING 005: Request(s) to plot nonexisting functions were encoutered');
                PlotFunctionAvgBuf=PlotFunctionAvgBuf(PlotFunctionAvgBuf>0);
            end
        end
    
    elseif strcmp(PName,'PlotFunctionObservations') % common
        if strcmp(PValue{1},'yes')
            PlotFunctionObs=1;
        elseif strcmp(PValue{1},'no')
            PlotFunctionObs=0;
        else
            disp('SQG ERROR 016: PlotFunctionObservations takes values ''yes'' or ''no'' ');
            illegalInput=1;
            return
        end
         
    elseif strcmp(PName,'FunctionReferenceValue') % common, must be put in exact order as requested functions in PlotFunctionAverage
        RefFunctionValueBuf=cell(1,length(PValue));
        for j=1:length(PValue)
            if isnumeric(PValue{j})
                RefFunctionValueBuf{j}=PValue{j};
            else
                disp('SQG ERROR 017: Function reference values for plotting must be numeric');
                illegalInput=1;
                return
            end
        end

    elseif strcmp(PName,'IndexesPlottedVariables')
        if NumProblems ==1
            buf=zeros(1,length(PValue));
            for j=1:length(PValue)
                if isnumeric(PValue{j}) && (PValue{j} <= nVars) && (PValue{j} >= 1) && ...
                        (floor(PValue{j}) == PValue{j}) && (length(PValue{j}) == 1)           
                    buf(j)=PValue{j};
                else
                    disp(['SQG ERROR 020: Indexes of plotted variables must be integers between 1 and ' num2str(nVars(1))]);
                    illegalInput=1;
                    return
                end
            end
            PlotVariableNumber{1}=buf;
            ProblemCorrespondence=1;
        else 
            ProblemCorrespondence=zeros(1,length(PValue));
            for j=1:length(PValue)
                PVJ=PValue{j};
                if ~iscell(PVJ)
                     disp('SQG ERROR 043: Illegal composition of the list of variables to plot');
                     illegalInput=1;
                     return
                end
                if isnumeric(PVJ{1}) && (length(PVJ{1}) == 1) && min((floor(PVJ{1}) == PVJ{1}))  ...
                        && (PVJ{1} >=1) && (PVJ{1} <= NumProblems) 
                    ProblemOfPlottedVariables=PVJ{1};
                    ProblemCorrespondence(j)=ProblemOfPlottedVariables;
                elseif ischar(PVJ{1})
                    WrongName=1;
                    for k=1:NumProblems
                        if strcmp(PVJ{1},ProblemNames{k})
                            ProblemOfPlottedVariables=k;
                            ProblemCorrespondence(j)=ProblemOfPlottedVariables;
                            WrongName=0;
                            break
                        end
                    end
                    if WrongName
                        disp('SQG ERROR 044: Illegal problem name is encountered');
                        illegalInput=1;
                        return
                    end
                    buf=zeros(1,length(PVJ)-1);
                    for jj=2:length(PVJ)
                        if isnumeric(PVJ{jj}) && (PVJ{jj} <= nVars(ProblemOfPlottedVariables)) ...
                                && (PVJ{jj} >= 1) && (floor(PVJ{jj}) == PVJ{jj}) && (length(PVJ{jj}) == 1)           
                            buf(jj-1)=PVJ{jj};
                        else
                            disp(['SQG ERROR 020a: Indexes of plotted variables must be integers between 1 and ' num2str(nVars(1))]);
                            illegalInput=1;
                            return
                        end
                    end
                    PlotVariableNumber{ProblemOfPlottedVariables}=buf;
                else
                    disp('SQG ERROR 043: Illegal composition of the list of variables to plot');
                    illegalInput=1;
                    return
                end
            end
        end

 elseif strcmp(PName,'ReferencesPlottedVariables')
        if NumProblems ==1
            buf=zeros(1,length(PValue));
            for j=1:length(PValue)
                if isnumeric(PValue{j}) && ~isempty(PValue{j}) && (length(PValue{j}) == 1) ...
                        && ~isinf(PValue{j})           
                    buf(j)=PValue{j};
                else
                    disp('SQG ERROR 021: References for plotted variables must have numeric values');
                    illegalInput=1;
                    return
                end
            end    
            RefBuf={buf};
        else
            if ~iscell(PValue)
                disp('SQG ERROR 045: Illegal composition of the list of references to variables');
                illegalInput=1;
                return
            end
            RefBuf=cell(1,length(PValue));
            for j=1:length(PValue)       
                PV=PValue{j};
                if isempty(PV)
                    continue
                end
                buf=zeros(1,length(PV));
                for jj=1:length(PV)
                    if isnumeric(PV{jj}) && ~isempty(PV{jj}) && (length(PV{jj}) == 1) ...
                            && ~isinf(PV{jj})          
                        buf(jj)=PV{jj};
                    else
                        disp('SQG ERROR 021a: References for plotted variables must have numeric values');
                        illegalInput=1;
                        return
                    end
                end    
                RefBuf{j}=buf;
            end
        end
        
        elseif strcmp(PName,'PlotConstraintViolation')  % common
        if (NumProblems == 1) && (length(PValue) == 1)
            if strcmp(PValue{1},'yes')
                PlotConBuf=1;
            elseif strcmp(PValue{1},'no')
                PlotConBuf=[];
            else
                disp('SQG ERROR 315: PlotConstraintViolation with a single objective takes values ''yes'' or ''no'' ');
                illegalInput=1;
                return
            end  
        else    
            PlotConBuf=zeros(1,length(PValue));
            for j=1:length(PValue)
                if isnumeric(PValue{j}) && ~isinf(PValue{j}) && (floor(PValue{j}) == PValue{j}) && ...
                    (PValue{j} <= NumProblems) && (PValue{j} > 0)
                    PlotConBuf(j)=PValue{j};
                elseif ischar(PValue{j})
                    for k=1:NumProblems
                        if strcmp(PValue{j},ProblemNames{k})
                            PlotConBuf(j)=k;
                            break
                        end
                    end
                else
                    disp('SQG ERROR 342: Illegal request for plotting constraint violation');
                    illegalInput=1;
                    return
                end
            end
            if sum(PlotConBuf >0) < length(PValue)
                disp('SQG WARNING 305: Request(s) to plot violation of nonexisting constraints were encoutered');
                PlotConBuf=PlotConBuf(PlotConBuf>0);
            end
        end
   
    elseif strcmp(PName,'SaveTraceFile')
        if ischar(PValue{1})       
            SaveFileName=PValue{1};
        else
            disp('SQG ERROR 022: File name must be a string');
            illegalInput=1;
            return
        end
        
    elseif strcmp(PName,'ExperimentDescription')
        if ischar(PValue{1}) 
            RunComment=PValue{1};
        else
            disp('SQG ERROR 023: Experiment description must be a string');
            illegalInput=1;
            return
        end
        
    elseif strcmp(PName,'SaveFigures')
        if ischar(PValue{1}) 
            SaveFigures=PValue{1};
        else
            disp('SQG ERROR 423: Header of figures name must be a string');
            illegalInput=1;
            return
        end
               
    elseif strcmp(PName,'InitializeRandomGenerator')
        if strcmp(PValue{1},'yes')
            randInitFlag=1;
        elseif strcmp(PValue{1},'no')
            randInitFlag=0;
        else
            disp('SQG ERROR 026: InitializeRandomGenerator takes values ''yes'' or ''no'' ');
            illegalInput=1;
            return
        end
        
    elseif strcmp(PName,'ProblemNames')
        continue
        
    elseif strcmp(PName,'ProblemTypes')
        continue
        
    elseif strcmp(PName,'ProblemDimensions')
        continue
        
    else
        disp(['SQG WARNING 001: unknown parameter ' PName ' is ignored']);   
    end
    
end
  
% Process parameters

iavg = floor(iter*FractionLastAveragingIterations);
iavg(iavg == 0) =1;

if DefaultFiniteDifferences
    FDiff=cell(1,NumProblems);
    for i=1:NumProblems
        FDiff{i}=AccuracyVariables{i}*AccuracyToFinDiff;
    end
end

if DefaultTraceAveraging
    TraceAveraging(:,1)=1/floor(iter*FractionForTraceAveraging);
end

if DefaultConstraintsAveraging
    SecondariesAveraging(:,1)=1/floor(iter*FractionForTraceAveraging);
end

if PlotFunctionObs && isempty(PlotFunctionAvgBuf)
    disp('SQG WARNING 006: Request to plot function observations without function averages is ignored')
    PlotFunctionObs=0;
end

for i=1:length(PlotFunctionAvgBuf)
       PlotFunctionAvg(PlotFunctionAvgBuf(i))=1;
       if ~isempty(RefFunctionValueBuf)
           RefFunctionValue{PlotFunctionAvgBuf(i)}=RefFunctionValueBuf{i};
       end
end
if (length(PlotFunctionAvgBuf) ~= length(RefFunctionValueBuf)) &&  ~isempty(PlotFunctionAvgBuf) && ~isempty(RefFunctionValueBuf)
    disp('SQG WARNING 007: The length of function references is different from the number of plotted functions')
end

if length(ProblemCorrespondence) == length(RefBuf) 
    for i=1:length(RefBuf)
        if ~isempty(RefBuf{i})
            if length(PlotVariableNumber{ProblemCorrespondence(i)}) == length(RefBuf{i})
                RefVariablesValues{ProblemCorrespondence(i)}=RefBuf{i};
            else
                disp('SQG WARNING 002: Variable references ignored because their list is incomplete')
            end
        end
    end
elseif ~isempty(RefBuf)
    disp('SQG WARNING 002: Variable references ignored because the number of reference values is different from number of plotted variables')
end

if DefaultProblemScales
    for i=1:NumProblems
        ProblemScale{i}=ones(1,length(schedule{i})+1);
    end
else
    for i=1:NumProblems
        buf=ProblemScale{i};
        nn=length(schedule{i})+1;
        if length(buf) > nn
            buf=buf(1:nn);
        elseif length(buf) < nn
            buf=[buf buf(end)*ones(1,nn-length(buf))]; %#ok<AGROW>
        end
        ProblemScale{i}=buf;
    end
end

% Checking if we have constraints

maxCon=max(ConNumber);
if maxCon == 0
    ConTypes = [];
    ConRHS = [];
    ConRef = [];
    Secondaries = [];
    DSecondaries = [];
    EstimateGradient=0;
else
    ConFunctions = ConFunctions(:,1:maxCon);
    ConTypes = ConTypes(:,1:maxCon);
    ConRHS = ConRHS(:,1:maxCon);
    ConDirection = ConDirection(:,1:maxCon);
    ConRef = ConRef(:,1:maxCon);
    Secondaries= Secondaries(:,1:maxCon);
    DSecondaries= DSecondaries(:,1:maxCon);
    
    % Extracting secondary functions from description of constraints

    for k=1:NumProblems
        for i=1:ConNumber(k)
            nF=ConFunctions(k,i);
            NewFunction=1;
            for j=1:NSecondaries(i)
                if nF == Secondaries(k,j)
                    if ConDirection(k,i) ~= DSecondaries(k,j)
                        disp(['SQG ERROR 222: Contradictory definition of direction for function ' ...
                            num2str(nF) ' of problem ' num2str(k)]);
                        illegalInput=1;
                        return
                    end
                    NewFunction=0;
                    break
                end       
            end
            if NewFunction
                NSecondaries(k)=NSecondaries(k)+1;
                Secondaries(k,NSecondaries(k))=nF;
                DSecondaries(k,NSecondaries(k))=ConDirection(k,i);
            end
        end
    end

    % Ordering of the list of secondary functions

    for i=1:NumProblems
        [Secondaries(i,1:NSecondaries(i)),IX]=sort(Secondaries(i,1:NSecondaries(i)));
        DSecondaries(i,1:NSecondaries(i))=DSecondaries(i,IX);
    end

    % Reference from constraint list to the list of secondaries 

    for i=1:NumProblems
        for ii=1:ConNumber(i)
            for j=1:NSecondaries(i)
                if ConFunctions(i,ii) == Secondaries(i,j)
                    ConRef(i,ii)=j; 
                end
            end
        end
    end
    
    % Find if a function has secondary role too
    
    for i =1:NumProblems
        for j=1:NSecondaries(i)
            IsSecondary(Secondaries(i,j))=1;
        end
    end
    
    % Form the list of problems where to draw constraint violations
    
    for i=1:length(PlotConBuf)
        if ConNumber(PlotConBuf(i)) > 0
            PlotConstraintsViolation(PlotConBuf(i))=1;
        else
            disp('SQG WARNING 302: Request to plot constraints violation for problem without expectation constraints is ignored')
        end
    end
    
    for i=1:NumProblems
        if PenaltyNotSet && ConProcessing(i) 
            ConPenalty(i)=1;
        end
    end
    
end

end

