% Simple implementation of ACOPF with SQG version 5 

%% Simple 4 bus case as illustration and starting point
%  See fiugre for picture and parameters
%  ----------------------------------------------------
%%
close all
clear
clc

%How may timesteps?
T=4; %1,4

% Solves the test problem min E[Cgen(Pgen)] with SQG where wind power  Pw 
% is distributed uniformly around windMean on the range [(1-wRange)z (1+wRange)z] 
% where z is an arbitrary element of wind.

%% Simple case where only one step

% 1. Setting the test problem
if T ==1
% T = 1.000;

windMean=30;
rRange = 0.5;

%used to calc g and w_reaised
windProjection = 25;

%x=[Pg];
x0=100+20-windMean;
parConstr = {[50] [110]};






% 1.2 Setting SQG parameters
%parFun(Nagents,T,SOC,priceMean,rRange)
parFun={T windProjection windMean rRange};
    
parAlg={
  { 'ProblemNames' 'ACOPF wind test' }
    { 'ProblemDimensions' 1 }
    { 'ProblemTypes' 'min' }  
    { 'StepMultiplier' 0.5}  
    { 'NumberIterations' 300 }
    { 'InitialStep' 0.035 }    
    { 'PlotFunctionAverage' 'yes'}
    { 'PlotFunctionObservations' 'yes'}
    { 'IndexesPlottedVariables' 1}
    %{ 'IndexesPlottedVariables' 1 2 3 4}
    { 'SaveTraceFile' 'QuadraticTest_01'}
    { 'StepFiniteDifferences' 0.001}
    %{ 'FunctionReferenceValue' fSol }
    %{ 'ReferencesPlottedVariables' xSol(1)}
    %{ 'ReferencesPlottedVariables' xSol(1) xSol(2) xSol(3) xSol(4)}
    { 'AveragingFiniteDifferences' 1}
    { 'ExperimentDescription' 'Produced by tryAgentProbelm'}
    };

%parConstr = {[50 50 50 50] [110 110 110 110]};

%x0=[Pg_t1  Pg_t2 Pg_t3 Pg_t4];
%windProjection = [30   40   20   30];

%x0=[80 90 70 80];

% %windProjection = guesses
% windProjection = [30   40   20   30];
% %Nagents = 4;


%xSol=[1.0000    1.0000    1.0000    1.0000]';                     % solution without stochastic
%fSol=-3.5031e+05;




% 1.3 Calling SQG

tic

[xOut,fval,exitFlag,InfIter]=SQG_v05(@SimulateACOPF_Opt_v01,parFun,parAlg,x0,@ACOPFsimBounds,parConstr);

tOpt=toc

end


%% Simple case where four time steps

% 1. Setting the test problem
if T ==4
% T = 1.000;

windMean= [30 20 50 30];
rRange = 0.25;

%used to calc g and w_reaised
windProjection = [35 35 35 35]; %??

%x=[Pg];
x0= 100+20-windMean;
parConstr = {[50 50 50 50] [110 110 110 110]};


%x0=[Pg_t1  Pg_t2 Pg_t3 Pg_t4];
%windProjection = [30   40   20   30];

%x0=[80 90 70 80];

% %windProjection = guesses
% windProjection = [30   40   20   30];




% 1.2 Setting SQG parameters
%parFun(Nagents,T,SOC,priceMean,rRange)
parFun={T windProjection windMean rRange};
    
parAlg={
  { 'ProblemNames' 'ACOPF wind test' }
    { 'ProblemDimensions' 4 }
    { 'ProblemTypes' 'min' }  
    { 'StepMultiplier' 0.5}  
    { 'NumberIterations' 300 }
    { 'InitialStep' 0.035 }    
    { 'PlotFunctionAverage' 'yes'}
    { 'PlotFunctionObservations' 'yes'}
    %{ 'IndexesPlottedVariables' 1}
    { 'IndexesPlottedVariables' 1 2 3 4}
    { 'SaveTraceFile' 'QuadraticTest_01'}
    { 'StepFiniteDifferences' 0.001}
    %{ 'FunctionReferenceValue' fSol }
    %{ 'ReferencesPlottedVariables' xSol(1)}
    %{ 'ReferencesPlottedVariables' xSol(1) xSol(2) xSol(3) xSol(4)}
    { 'AveragingFiniteDifferences' 1}
    { 'ExperimentDescription' 'Produced by tryAgentProbelm'}
    };



%xSol=[1.0000    1.0000    1.0000    1.0000]';                     % solution without stochastic
%fSol=-3.5031e+05;




% 1.3 Calling SQG

tic

[xOut,fval,exitFlag,InfIter]=SQG_v05(@SimulateACOPF_Opt_v01,parFun,parAlg,x0,@ACOPFsimBounds,parConstr);

tOpt=toc

end



% 
% fileID = fopen('results.txt','w');
% fprintf(fileID, '%10s %8s %8s %8s %8s %8s\n','fval','tOpt','xOut(1)','xOut(2)','xOut(3)','xOut(4)');
% fprintf(fileID, '%10.0f %2.6f %2.6f %2.6f %2.6f %2.6f',fval,tOpt,xOut(1),xOut(2),xOut(3),xOut(4));
% fclose(fileID);
% 
% parAlg={
%     { 'ProblemNames' 'Traditional' 'Renewable' }
%     { 'ProblemDimensions' 1 }
%     { 'ProblemTypes' 'max'}  
%     { 'StepMultiplier' 0.5}  
%     { 'NumberIterations' 20000 }
%     { 'InitialStep' 0.0005  0.002 } 
%     { 'InitializeRandomGenerator' 'yes'}
% %     { 'PlotFunctionAverage' 1 2 3 4 5 6}
% %     { 'PlotFunctionObservations' 'yes'}
%      { 'IndexesPlottedVariables' {'Traditional' 1} {'Renewable' 1}  }
% %     { 'SaveTraceFile' 'SQG_NashEx_set2_01TraceAveraging'}
%     { 'StepFiniteDifferences' 0.005}
% %     { 'FunctionReferenceValue' ProfProd1(NashN1,NashN2) ProfProd1(NashN1,NashN2) ProfProd2(NashN1,NashN2) ProfProd2(NashN1,NashN2) }
% %     { 'ReferencesPlottedVariables' {Grid1(NashN1)} {Gridy(NashN2)}  }
%     { 'AveragingFiniteDifferences' 0.1}
%     { 'TraceAveraging' [1 0 1 ]}
% %     { 'PlotType' 'Path'}
% %     { 'ExperimentDescription' 'Produced by TrySimulateEnergyOptSQG_v02, data set 1'}
%     };
% 
% 
% % parConstr = {[0.5;0] [0.5;0] [0.35;0] [0.35;0] }; % f range
% % parConstr = {[0.5;0] [0.5;0] [0.25;0] [0.25;0] }; % c range
% % parConstr = {[0.3;0] [0.3;0] [0.2;0] [0.2;0] }; % d range
%  parConstr = {[0.5;0] [0.5;0] };
% 
%  x0=[0.1; 0.1];
% 
% UnL=1.0;
% UnL1=1.0;
% UnL2=1.0;
% parFun=[Experiment UnL UnL1 UnL2];
% 
% tic
% 
% [xOut, fval, exitFlag, InfIter] = SQG_v05_1(@SimulateEnergyOpt_v02, parFun, parAlg, x0, @Bounds_mex, ...
%         parConstr);
% 
% 
% ElapsedTime=toc %#ok<NOPTS>
% 
% [T, S, a1_, b1_, a2_, b2_, h_, U, cBar_, c_, Zmax_, r1_, r2_, r3_, Q0, theta, g_, beta]=...
%         getProblemData(parFun);
% 
%     
%     
%     
%     clear all
% 
% MexMode=1;
% Mode=1;
% parFun=[Mode MexMode];
% 
% parAlg={
%   { 'ProblemNames' 'Profit' }
%     { 'ProblemDimensions' 1 }
%     { 'ProblemTypes' 'max' }  
%     { 'StepMultiplier' 0.5}  
%     { 'NumberIterations' 5000 }
%     { 'InitialStep' 0.0004 }    
%     { 'InitializeRandomGenerator' 'yes'}
%     { 'PlotFunctionAverage' 'yes'}
%     { 'PlotFunctionObservations' 'yes'}
%     { 'IndexesPlottedVariables' 1}
%     { 'SaveTraceFile' 'SQG_Trace_01_test2c'}
%     { 'StepFiniteDifferences' 0.0025}
%     { 'FunctionReferenceValue' 86.1075648119054 }
%     { 'ReferencesPlottedVariables' 0.0638740000000000 }
%     { 'AveragingFiniteDifferences' 0.2}
%     { 'ExperimentDescription' 'Produced by TrySimulateEnergyOpt_v01, data set 1, Mode 1'}
%     };
% 
% %load
% 
% parConstr = [0.5 ; 0]; %upper and lower bound p? parameter - data for funskjonen som relateres til beskrankninger
% x0=0.4;
% 
% tic
% 
% [xOut, fval, exitFlag, InfIter] = SQG_v05(@SimulateEnergyOpt_v01, parFun, parAlg, x0, @Bounds_mex, ...
%     parConstr);
% 
% ElapsedTime=toc %#ok<NOPTS>
% 
% % SQG_Trace_01_test2b - very good for Mode 0




%% ===== Not Used Here =======
% 1.1 Draw the two dimensional graph. It is done for pure demonstrational
% purposes and is not required for optimization

% if (length(SOC) ~= 2) && (drawFunctionGraph == 1)
%     disp('Mesh graph can be drawn only for two dimensional problem')
%     drawFunctionGraph = 0;
% end
% if drawFunctionGraph
%      xRange = [1 2];
%      yRange = [0 1];
%      nGrid=1001;
%      xD=(xRange(2)-xRange(1))/(nGrid-1);
%      yD=(yRange(2)-yRange(1))/(nGrid-1);
%      xGrid=xRange(1):xD:xRange(2);
%      yGrid=yRange(1):yD:yRange(2);
%      for i1=1:nGrid
%          for i2=1:nGrid
%              x=[xGrid(i1) yGrid(i2)]';
%              Q(i1,i2)=x'*Amean*x/2+bmean'*x+cmean;
%          end
%      end
%      mesh(xGrid,yGrid,Q);
%     
% end
% 