% Simple implementation of ACOPF with SQG version 5 

%% Simple 4 bus case as illustration and starting point
%  See fiugre for picture and parameters
%  ----------------------------------------------------
%%
close all
clear
clc

%Which ACOPF case?
ACOPFcase = 1; % 4BusCase
numbus = 4;

TotEnBatMax = 50;
TotEnBat0 = TotEnBatMax/2;

%How may timesteps?
T=4; %1,4

% Solves the test problem min E[Cgen(Pgen)] with SQG where wind power  Pw 
% is distributed uniformly around windMean on the range [(1-wRange)z (1+wRange)z] 
% where z is an arbitrary element of wind.

%% Simple case where only one step

% 1. Setting the test problem
if T ==1
% T = 1.000;

windMean=30;
rRange = 0.25;

%used to calc g and w_reaised
windProjection = 25; %not used I believe

%x=[Pg];
% x tries to find how much battery to store
x0=80+windMean-100;
%x0=100-80-windMean;
parConstr = {[(x0-windMean)] [(x0+windMean)]};






% 1.2 Setting SQG parameters
%parFun(Nagents,T,SOC,priceMean,rRange)
parFun={ACOPFcase T windProjection windMean rRange TotEnBatMax};
    
parAlg={
  { 'ProblemNames' 'ACOPF wind test' }
    { 'ProblemDimensions' 1 }
    { 'ProblemTypes' 'min' }  
    { 'StepMultiplier' 0.5}  
    { 'NumberIterations' 100 }
    { 'InitialStep' 0.035 }    
    { 'PlotFunctionAverage' 'yes'}
    { 'PlotFunctionObservations' 'yes'}
    { 'IndexesPlottedVariables' 1}
    %{ 'IndexesPlottedVariables' 1 2 3 4}
    { 'SaveTraceFile' 'ACOPF4busTest_01'}
    { 'StepFiniteDifferences' 0.001}
    %{ 'FunctionReferenceValue' fSol }
    %{ 'ReferencesPlottedVariables' xSol(1)}
    %{ 'ReferencesPlottedVariables' xSol(1) xSol(2) xSol(3) xSol(4)}
    { 'AveragingFiniteDifferences' 1}
    { 'ExperimentDescription' 'Produced by Stochastic ACOPF'}
    };

%parConstr = {[50 50 50 50] [110 110 110 110]};

%x0=[Pg_t1  Pg_t2 Pg_t3 Pg_t4];
%windProjection = [30   40   20   30];

%x0=[80 90 70 80];

% %windProjection = guesses
% windProjection = [30   40   20   30];
% %Nagents = 4;


%xSol=[1.0000    1.0000    1.0000    1.0000]';                     % solution without stochastic
%fSol=-3.5031e+05;




% 1.3 Calling SQG

tic

[xOut,fval,exitFlag,InfIter]=SQG_v05(@SimulateACOPF_Opt_v02,parFun,parAlg,x0,@ACOPFsimBounds,parConstr);

tOpt=toc

xOut
end


%% Simple case where four time steps

% 1. Setting the test problem
if T ==4
% T = 1.000;

windMean= [30 20 50 30];
rRange = 0.25;

%used to calc g and w_reaised
windProjection = [35 35 35 35]; %??

% x tries to find how much battery to store
x0=80+windMean-100;
%x0 = [10 0 30 10]
%x0=100-80-windMean;

% OBS, these need to be updated so that they enforce charging max min.
%
%parConstr = {[x0-windMean] [x0+windMean]};
%parConstr = {[-windMean] [+windMean]}; used last befor dymamic bounds

parConstWithBat = {[-windMean] [+windMean] TotEnBatMax TotEnBat0};

parConstrBat = {TotEnBatMax TotEnBat0};
%ACOPFdynamicSimBounds(x0,parConstrBat)

%parConstr = {[50 50 50 50] [110 110 110 110]};


%x0=[Pg_t1  Pg_t2 Pg_t3 Pg_t4];
%windProjection = [30   40   20   30];

%x0=[80 90 70 80];

% %windProjection = guesses
% windProjection = [30   40   20   30];




% 1.2 Setting SQG parameters
%parFun(Nagents,T,SOC,priceMean,rRange)
parFun={ACOPFcase T windProjection windMean rRange TotEnBatMax};
    
parAlg={
  { 'ProblemNames' 'ACOPF wind test' }
    { 'ProblemDimensions' 4 }
    %{ 'ProblemStates' numbus*4 } %??
    { 'ProblemTypes' 'min' } 
    { 'StepMultiplier' 0.2 }
    %{ 'StepMultiplier' 0.2 0.2 0.2 0.2}  %works as the one above
    { 'StepDirections' 'grad'}
    
    { 'NumberIterations' 100}
    { 'InitialStep' 0.035 }
    %{ 'InitialStep' 0.035 0.035 0.035 0.035 } %works as the one above
    { 'PlotFunctionAverage' 'yes'}
    { 'PlotFunctionObservations' 'yes'}
    %{ 'IndexesPlottedVariables' 1}
    { 'IndexesPlottedVariables' 1 2 3 4}
    { 'SaveTraceFile' 'ACOPF4busTest_04'}
    %{ 'StepFiniteDifferences' 0.001}
    
    %{ 'FunctionReferenceValue' fSol }
    %{ 'ReferencesPlottedVariables' xSol(1)}
    %{ 'ReferencesPlottedVariables' xSol(1) xSol(2) xSol(3) xSol(4)}
    
    { 'AveragingFiniteDifferences' 0.1}
    %They define the moving average multipliers utilized for
    %getting the moving average estimate of the function observations at finite difference points
    %utilized to compute the finite differences. If such multiplier equals 0 then the new 			
    %observations are totally discarded and if it equals 1 then the old observations are totally 		
    %discarded. Default: ParameterValue = 0.001.
    { 'ExperimentDescription' 'Produced by Stochastic ACOPF'}
    };



%xSol=[1.0000    1.0000    1.0000    1.0000]';                     % solution without stochastic
%fSol=-3.5031e+05;




% 1.3 Calling SQG

tic

% shall manipulate ACOPFsimBounds and parConst to make it dynamic
%[xOut,fval,exitFlag,InfIter]=SQG_v05(@SimulateACOPF_Opt_v02,parFun,parAlg,x0,@ACOPFsimBounds,parConstr);
%[xOut,fval,exitFlag,stateVar]=SQG_v05(@SimulateACOPF_Opt_v02,parFun,parAlg,x0,@ACOPFsimBounds,parConstr);

%[xOut,fval,exitFlag,InfIter]=SQG_v05(@SimulateACOPF_Opt_v02,parFun,parAlg,x0,@ACOPFdynamicSimBounds,parConstrBat);
%some problems here with too lafre x values

[xOut,fval,exitFlag,InfIter]=SQG_v05(@SimulateACOPF_Opt_v02,parFun,parAlg,x0,@ACOPFdynamicSimBounds2,parConstWithBat);
%mistenker det kan v?re noe feil i dyn utregningen som for?rsaket de enorme
%ladeverdiene i forigen SimBounds

tOpt=toc
xOut

plotSQGsolutionOfACOPF(parFun,xOut);
xOut

end


%%  xOut is result
    %update control variable values
% 1 theta1
% 2 theta2
% 3 theta3
% 4 theta4
% 5 V_1
% 6 V_2
% 7 V_3
% 8 V_4
% 9 P_1
% 10 P_2
% 11 P_3
% 12 P_4
% 13 Q_1
% 14 Q_2
% 15 Q_3
% 16 Q_4

%     %% ----- evaluate constraints -----
%     %first display resulting V and angle
%     v_out = stateVar(5:8)*baseKV
%     angle_out = stateVar(1:4)*(pi/180)
%      
% %     %evaluate
% %     Vm_out = x_new(5:8);
% %     Va_out = x_new(1:4);
% %     
% %     % reconstruct V
% %     V = Vm_out .* exp(1j * Va_out);
% %     % evaluate power flow equations
% %     [Ybus, Yf, Yt] = makeYbus(baseMVA, bus, branch);
% %     Sbus = makeSbus(baseMVA, bus, gen)
% %     %resulteing injections
% %     S = V .* conj(Ybus * V)*baseMVA
%     %diff is ceq
%     %DiffPowerFlow = [ceq(1:4) ceq(5:8)]*baseMVA
%  % ----  Find generated power
%     Pgen      = stateVar(9:12)*baseMVA;
%     Qgen      = stateVar(13:16)*baseMVA;
%     
% %     Ploa1     = Pd(1);
% %     Ploa2     = Pd(2);
% %     Ploa3     = Pd(3);
% %     Ploa4     = Pd(4);
%     
% %     
% %     for i=1:length(x_new)/num_var
% %         State_ofC(i)= x_new(21+(i-1)*26)*baseMVA;
% %         gen1(i)=x_new(19+(i-1)*26)*baseMVA;
% %         gen2(i)=x_new(20+(i-1)*26)*baseMVA;
% %         charge(i)=x_new(25+(i-1)*26)*baseMVA;
% %         discharge(i)=x_new(26+(i-1)*26)*baseMVA;
% %         Ploa1(i)=Pd(5,i);
% %         Ploa2(i)=Pd(7,i);
% %         Ploa3(i)=Pd(9,i);
% %     end
% 
% figure('Name','Generator results','Numbertitle','off');
% c_gen = categorical({'gen1','gen2','gen3','gen4'});
% gen_vals = [Pgen Qgen]
% bar(c_gen,gen_vals);
% xlabel('Generator')
% ylabel('Generated power')
% legend('Active power, P [MW]','Reactive power, Q [MVAr]')
% title('Results for the generators')
% 
% figure('Name','Load demand','Numbertitle','off');
% % c_load = categorical({'P1','P2','P3','P4'});
% % Ploads = [Ploa1 Ploa2 Ploa3 Ploa4]
% c_load = categorical({'1','2','3','4'});
% Ploads = [Pd Qd]
% bar(c_load,Ploads)
% xlabel('Bus number')
% ylabel('Load values')
% legend('Active power, P [MW]','Reactive power, Q [MVAr]')
% title('Demand for the bus loads')


%%

% 
% fileID = fopen('results.txt','w');
% fprintf(fileID, '%10s %8s %8s %8s %8s %8s\n','fval','tOpt','xOut(1)','xOut(2)','xOut(3)','xOut(4)');
% fprintf(fileID, '%10.0f %2.6f %2.6f %2.6f %2.6f %2.6f',fval,tOpt,xOut(1),xOut(2),xOut(3),xOut(4));
% fclose(fileID);
% 
% parAlg={
%     { 'ProblemNames' 'Traditional' 'Renewable' }
%     { 'ProblemDimensions' 1 }
%     { 'ProblemTypes' 'max'}  
%     { 'StepMultiplier' 0.5}  
%     { 'NumberIterations' 20000 }
%     { 'InitialStep' 0.0005  0.002 } 
%     { 'InitializeRandomGenerator' 'yes'}
% %     { 'PlotFunctionAverage' 1 2 3 4 5 6}
% %     { 'PlotFunctionObservations' 'yes'}
%      { 'IndexesPlottedVariables' {'Traditional' 1} {'Renewable' 1}  }
% %     { 'SaveTraceFile' 'SQG_NashEx_set2_01TraceAveraging'}
%     { 'StepFiniteDifferences' 0.005}
% %     { 'FunctionReferenceValue' ProfProd1(NashN1,NashN2) ProfProd1(NashN1,NashN2) ProfProd2(NashN1,NashN2) ProfProd2(NashN1,NashN2) }
% %     { 'ReferencesPlottedVariables' {Grid1(NashN1)} {Gridy(NashN2)}  }
%     { 'AveragingFiniteDifferences' 0.1}
%     { 'TraceAveraging' [1 0 1 ]}
% %     { 'PlotType' 'Path'}
% %     { 'ExperimentDescription' 'Produced by TrySimulateEnergyOptSQG_v02, data set 1'}
%     };
% 
% 
% % parConstr = {[0.5;0] [0.5;0] [0.35;0] [0.35;0] }; % f range
% % parConstr = {[0.5;0] [0.5;0] [0.25;0] [0.25;0] }; % c range
% % parConstr = {[0.3;0] [0.3;0] [0.2;0] [0.2;0] }; % d range
%  parConstr = {[0.5;0] [0.5;0] };
% 
%  x0=[0.1; 0.1];
% 
% UnL=1.0;
% UnL1=1.0;
% UnL2=1.0;
% parFun=[Experiment UnL UnL1 UnL2];
% 
% tic
% 
% [xOut, fval, exitFlag, InfIter] = SQG_v05_1(@SimulateEnergyOpt_v02, parFun, parAlg, x0, @Bounds_mex, ...
%         parConstr);
% 
% 
% ElapsedTime=toc %#ok<NOPTS>
% 
% [T, S, a1_, b1_, a2_, b2_, h_, U, cBar_, c_, Zmax_, r1_, r2_, r3_, Q0, theta, g_, beta]=...
%         getProblemData(parFun);
% 
%     
%     
%     
%     clear all
% 
% MexMode=1;
% Mode=1;
% parFun=[Mode MexMode];
% 
% parAlg={
%   { 'ProblemNames' 'Profit' }
%     { 'ProblemDimensions' 1 }
%     { 'ProblemTypes' 'max' }  
%     { 'StepMultiplier' 0.5}  
%     { 'NumberIterations' 5000 }
%     { 'InitialStep' 0.0004 }    
%     { 'InitializeRandomGenerator' 'yes'}
%     { 'PlotFunctionAverage' 'yes'}
%     { 'PlotFunctionObservations' 'yes'}
%     { 'IndexesPlottedVariables' 1}
%     { 'SaveTraceFile' 'SQG_Trace_01_test2c'}
%     { 'StepFiniteDifferences' 0.0025}
%     { 'FunctionReferenceValue' 86.1075648119054 }
%     { 'ReferencesPlottedVariables' 0.0638740000000000 }
%     { 'AveragingFiniteDifferences' 0.2}
%     { 'ExperimentDescription' 'Produced by TrySimulateEnergyOpt_v01, data set 1, Mode 1'}
%     };
% 
% %load
% 
% parConstr = [0.5 ; 0]; %upper and lower bound p? parameter - data for funskjonen som relateres til beskrankninger
% x0=0.4;
% 
% tic
% 
% [xOut, fval, exitFlag, InfIter] = SQG_v05(@SimulateEnergyOpt_v01, parFun, parAlg, x0, @Bounds_mex, ...
%     parConstr);
% 
% ElapsedTime=toc %#ok<NOPTS>
% 
% % SQG_Trace_01_test2b - very good for Mode 0




%% ===== Not Used Here =======
% 1.1 Draw the two dimensional graph. It is done for pure demonstrational
% purposes and is not required for optimization

% if (length(SOC) ~= 2) && (drawFunctionGraph == 1)
%     disp('Mesh graph can be drawn only for two dimensional problem')
%     drawFunctionGraph = 0;
% end
% if drawFunctionGraph
%      xRange = [1 2];
%      yRange = [0 1];
%      nGrid=1001;
%      xD=(xRange(2)-xRange(1))/(nGrid-1);
%      yD=(yRange(2)-yRange(1))/(nGrid-1);
%      xGrid=xRange(1):xD:xRange(2);
%      yGrid=yRange(1):yD:yRange(2);
%      for i1=1:nGrid
%          for i2=1:nGrid
%              x=[xGrid(i1) yGrid(i2)]';
%              Q(i1,i2)=x'*Amean*x/2+bmean'*x+cmean;
%          end
%      end
%      mesh(xGrid,yGrid,Q);
%     
% end
% 